#include <ArduinoBLE_P.h>
/*
 * Copied from an existing sketch the relevant parts and modified / added what is necessary for the test.
 * To show how often the function readValue() is called, in the file BLELocalCharateristic() readValue(). 
 * 
 * Serial.print("read: ");
 * Serial.print(length);
 * Serial.print(" offset: ");
 * Serial.println(offset);
 * 
 * The sending data is 33 bytes long , but will be sliding one character left with each send.
 * 
 * I used 'nRF Connect' on my mobile as central.
 * 
 * This sketch waits for connect event. Then it sends a string. On the 'nrf_connect' I perform a read (down_arrow) once.  
 * It shows the string that was created by send_data before. With read (down_arrow) I see a new string.
 * 
 * As you can see from below 22 bytes are read everytime (based on the MTU size agreed with nrf_Connect)
 * 
Test. Compiled: 11:47:28

Connected event, central: 44:9f:94:89:e0:91
written: ABCDEFGHIJKLMNOPQRSTUVW0123456789         // send after connected the first time
length: 33
read: 22 offset: 0          // readValue() is called
read: 11 offset: 22         // readValue() is called again
CALLED!                     // NOW the call back is done
written: BCDEFGHIJKLMNOPQRSTUVW0123456789A        // send_data is called from the call back. one characters slided left
length: 33
read: 22 offset: 0
read: 11 offset: 22
CALLED!
written: CDEFGHIJKLMNOPQRSTUVW0123456789AB
length: 33
read: 22 offset: 0
read: 11 offset: 22
CALLED!
written: DEFGHIJKLMNOPQRSTUVW0123456789ABC
length: 33


Just for the fun for another test I have extended the string to 86 characters and also that works

Connected event, central: 4a:dd:1d:37:93:c
ABCDEFGHIJKLMNOPQRSTUVW0123456789!@#$%^&*()ABCDEFGHIJKLMNOPQRSTUVW0123456789!@#$%^&*()
read: 22 offset: 0
read: 22 offset: 22
read: 22 offset: 44
read: 20 offset: 66
CALLED!
BCDEFGHIJKLMNOPQRSTUVW0123456789!@#$%^&*()ABCDEFGHIJKLMNOPQRSTUVW0123456789!@#$%^&*()A

*/
/////////////////////////////////////////////////////////////////////////////////
// Performed modications ( 3 places)
/////////////////////////////////////////////////////////////////////////////////
/*
ArduinoBLE/src/local/BLELocalCharacteristic.h
ADD in private :   bool _Enable_BLEread_Callback;        // paulvha #issue 272

..............................................................................
ArduinoBLE/src/local/BLELocalCharacteristic.cpp

ADD : _Enable_BLEread_Callback(false), 

BLELocalCharacteristic::BLELocalCharacteristic(const char* uuid, uint16_t permissions, int valueSize, bool fixedLength) :
  BLELocalAttribute(uuid),
  _properties((uint8_t)(permissions&0x000FF)),
  _valueSize(min(valueSize, 512)),
  _valueLength(0),
  _fixedLength(fixedLength),
  _handle(0x0000),
  _broadcast(false),
  _written(false),
  _cccdValue(0x0000),
  _Enable_BLEread_Callback(false),                     // paulvha #issue 272
  _permissions((uint8_t)((permissions&0xFF00)>>8))
..............................................................................

ArduinoBLE/src/local/BLELocalCharacteristic.cpp

CHANGE readValue() to :

void BLELocalCharacteristic::readValue(BLEDevice device, uint16_t offset, uint8_t value[], int length)
{
  // data
  memcpy(value, _value + offset, length);

  if (_Enable_BLEread_Callback) {           // paulvha #issue 272

    if (offset + length >= _valueLength) {  // has all data been read then check for call back

       _Enable_BLEread_Callback = false;    // paulvha #issue 272

       if (_eventHandlers[BLERead]) {
            _eventHandlers[BLERead](device, BLECharacteristic(this));
       }
    }
  }
}

*/

const char BLE_PERIPHERAL_NAME[] = "Peripheral BLE";

// create the service
BLEService Service("19B10010-E8F2-537E-4F6C-D104768A1214");

// create characteristic and allow remote device to read and write
BLEStringCharacteristic rCharacteristic("19B10011-E8F2-537E-4F6C-D104768A1214", BLERead | BLEWrite, 100);

bool Onetime = true;
bool Connected = false;
String Test="ABCDEFGHIJKLMNOPQRSTUVW0123456789";
int8_t t_offset = 0;

/*
 * called when all reading has been done
 */
void iter_temp_log(BLEDevice *central, BLECharacteristic c) {
    Serial.println("CALLED!");
    Send_data();
}

/*
 * send a sliding string
 */
void Send_data(){
  int i,j;
  String t_send;
  
  j=t_offset;
  
  if (t_offset == Test.length()) t_offset=0;
  
  // for each letter
  for (i = 0; i < Test.length();i++){

   // copy from start offset 
   t_send += Test[j++];

   // loop around
   if (j == Test.length()) j = 0;
  }

  Serial.print("written: ");
  Serial.println(t_send);
  Serial.print("length: ");
  Serial.println(t_send.length());
  
  rCharacteristic.writeValue(t_send);
}

void setup() {
   Serial.begin(115200);
   delay(1000);
   Serial.printf("Test. Compiled: %s\n\r", __TIME__);

  ///////////////////////////////////////////////////
  // BLE INIT
  ///////////////////////////////////////////////////
  // begin initialization
  if (!BLE.begin()) {
    Serial.println(F("starting BLE failed!\r"));
    while (1);
  }

  // set the local name peripheral advertises
  BLE.setLocalName(BLE_PERIPHERAL_NAME);
  
  // set the UUID for the service this peripheral advertises:
  BLE.setAdvertisedService(Service);

  // add the characteristics to the service
  Service.addCharacteristic(rCharacteristic);

  // add the service
  BLE.addService(Service); 

  // set connection handlers
  BLE.setEventHandler(BLEConnected, blePeripheralConnectHandler);
  BLE.setEventHandler(BLEDisconnected, blePeripheralDisconnectHandler);

  // start advertising
  BLE.advertise();

  rCharacteristic.setEventHandler(BLERead, (BLECharacteristicEventHandler) iter_temp_log);
}

void loop() {
  
  // wait for connection
  while (! Connected){
    BLE.poll(); 
    delay(1000);     // don't hammer
  }
  
  if (Onetime){      // send first time after connect
    delay(1000);     // wait for central to settle
    Send_data();
    Onetime = false;
  }
  
  BLE.poll();
  delay(500);        // don't hammer
}

/**
 * called when central connects
 */
void blePeripheralConnectHandler(BLEDevice central) {
  // central connected event handler
  Serial.print("Connected event, central: ");
  Serial.println(central.address());
  Connected = true;
  Onetime = true;
  t_offset=0;
}

/**
 * called when central disconnects
 */
void blePeripheralDisconnectHandler(BLEDevice central) {
  // central disconnected event handler
  Serial.print("Disconnected event, central: ");
  Serial.println(central.address());
  Connected = false;      // block loop and wait for new connection
}
